@@ -15,17 +15,17 @@ from gaphor.diagram.presentation import (
     HandlePositionUpdate,
     Named,
     literal_eval,
+    text_name,
 )
-from gaphor.diagram.shapes import Box, IconBox, Text, ellipse, stroke
+from gaphor.diagram.shapes import Box, CssNode, IconBox, Text, ellipse, stroke
 from gaphor.diagram.support import represents
-from gaphor.i18n import gettext
-from gaphor.UML.recipes import stereotypes_str
+from gaphor.UML.compartments import text_stereotypes
 
 DEFAULT_JOIN_SPEC = "and"
 
 
 def no_movable_handles(item):
-    for h in item._handles:
+    for h in item.handles():
         h.movable = False
 
 
@@ -50,13 +50,11 @@ class InitialNodeItem(ActivityNodeItem, ElementPresentation):
         self.shape = IconBox(
             Box(draw=draw_initial_node),
             # Text should be left-top
-            Text(
-                text=lambda: stereotypes_str(self.subject),
-            ),
-            Text(text=lambda: self.subject and self.subject.name or ""),
+            text_stereotypes(self),
+            text_name(self),
         )
 
-        self.watch("subject[NamedElement].name")
+        self.watch("subject.name")
         self.watch("subject.appliedStereotype.classifier.name")
 
 
@@ -85,13 +83,11 @@ class ActivityFinalNodeItem(ActivityNodeItem, ElementPresentation):
         self.shape = IconBox(
             Box(draw=draw_activity_final_node),
             # Text should be right-bottom
-            Text(
-                text=lambda: stereotypes_str(self.subject),
-            ),
-            Text(text=lambda: self.subject and self.subject.name or ""),
+            text_stereotypes(self),
+            text_name(self),
         )
 
-        self.watch("subject[NamedElement].name")
+        self.watch("subject.name")
         self.watch("subject.appliedStereotype.classifier.name")
 
 
@@ -108,7 +104,7 @@ def draw_activity_final_node(_box, context, _bounding_box):
     ellipse(cr, 0, 0, d, d)
     cr.set_line_width(0.01)
     cr.set_line_width(2)
-    stroke(context)
+    stroke(context, fill=True)
 
     d = inner_radius * 2
     ellipse(cr, 5, 5, d, d)
@@ -131,13 +127,11 @@ class FlowFinalNodeItem(ActivityNodeItem, ElementPresentation):
         self.shape = IconBox(
             Box(draw=draw_flow_final_node),
             # Text should be right-bottom
-            Text(
-                text=lambda: stereotypes_str(self.subject),
-            ),
-            Text(text=lambda: self.subject and self.subject.name or ""),
+            text_stereotypes(self),
+            text_name(self),
         )
 
-        self.watch("subject[NamedElement].name")
+        self.watch("subject.name")
         self.watch("subject.appliedStereotype.classifier.name")
 
 
@@ -146,7 +140,7 @@ def draw_flow_final_node(_box, context, bounding_box):
     r = 10
     d = r * 2
     ellipse(cr, *bounding_box)
-    stroke(context)
+    stroke(context, fill=True)
 
     dr = (1 - math.sin(math.pi / 4)) * r
     cr.move_to(dr, dr)
@@ -167,31 +161,29 @@ class DecisionNodeItem(ActivityNodeItem, ElementPresentation):
         self.shape = IconBox(
             Box(draw=draw_decision_node),
             # Text should be left-top
-            Text(
-                text=lambda: stereotypes_str(self.subject),
-            ),
-            Text(text=self.node_type, style={"font-size": "small"}),
-            Text(text=lambda: self.subject and self.subject.name or ""),
+            text_stereotypes(self),
+            CssNode("type", None, Text(text=self.node_type)),
+            text_name(self),
         )
 
-        self.watch("show_underlaying_type")
-        self.watch("subject[NamedElement].name")
+        self.watch("show_underlying_type")
+        self.watch("subject.name")
         self.watch("subject.appliedStereotype.classifier.name")
 
-    show_underlaying_type: attribute[int] = attribute("show_type", int, 0)
+    show_underlying_type: attribute[int] = attribute("show_underlying_type", int, 0)
     combined: relation_one[UML.ControlNode] = association(
         "combined", UML.ControlNode, upper=1
     )
 
     def node_type(self):
-        if not self.show_underlaying_type:
+        if not self.show_underlying_type:
             return ""
         if self.combined:
-            return gettext("merge/decision")
+            return self.diagram.gettext("merge/decision")
         elif isinstance(self.subject, UML.MergeNode):
-            return gettext("merge")
+            return self.diagram.gettext("merge")
         elif isinstance(self.subject, UML.DecisionNode):
-            return gettext("decision")
+            return self.diagram.gettext("decision")
         return ""
 
 
@@ -206,10 +198,11 @@ def draw_decision_node(_box, context, _bounding_box):
     cr.line_to(r2, r * 2)
     cr.line_to(0, r)
     cr.close_path()
-    stroke(context)
+    stroke(context, fill=True)
 
 
 @represents(UML.ForkNode)
+@represents(UML.JoinNode)
 class ForkNodeItem(Named, Presentation[UML.ForkNode], HandlePositionUpdate):
     """Representation of fork and join node."""
 
@@ -224,19 +217,21 @@ class ForkNodeItem(Named, Presentation[UML.ForkNode], HandlePositionUpdate):
 
         self._shape = IconBox(
             Box(draw=self.draw_fork_node),
-            Text(
-                text=lambda: stereotypes_str(self.subject),
-            ),
-            Text(text=lambda: self.subject and self.subject.name or ""),
-            Text(
-                text=lambda: isinstance(self.subject, UML.JoinNode)
-                and self.subject.joinSpec not in (None, DEFAULT_JOIN_SPEC)
-                and f"{{ joinSpec = {self.subject.joinSpec} }}"
-                or "",
+            text_stereotypes(self),
+            text_name(self),
+            CssNode(
+                "joinspec",
+                None,
+                Text(
+                    text=lambda: isinstance(self.subject, UML.JoinNode)
+                    and self.subject.joinSpec not in (None, DEFAULT_JOIN_SPEC)
+                    and f"{{ joinSpec = {self.subject.joinSpec} }}"
+                    or ""
+                ),
             ),
         )
 
-        self.watch("subject[NamedElement].name")
+        self.watch("subject.name")
         self.watch("subject.appliedStereotype.classifier.name")
         self.watch("subject[JoinNode].joinSpec")
 
@@ -290,6 +285,6 @@ class ForkNodeItem(Named, Presentation[UML.ForkNode], HandlePositionUpdate):
 
     def point(self, x, y):
         h1, h2 = self._handles
-        d, p = distance_line_point(h1.pos, h2.pos, (x, y))
+        d, _p = distance_line_point(h1.pos, h2.pos, (x, y))
         # Subtract line_width / 2
         return d - 3
