@@ -15,14 +15,12 @@ module.
 """
 
 from gaphor import UML
-from gaphor.core import gettext
 from gaphor.core.modeling.properties import attribute
-from gaphor.core.styling import JustifyContent
 from gaphor.diagram.presentation import Classified, ElementPresentation
-from gaphor.diagram.shapes import Box, Text, stroke
+from gaphor.diagram.shapes import Box, stroke
 from gaphor.diagram.support import represents
-from gaphor.diagram.text import FontWeight
-from gaphor.UML.classes.stereotype import stereotype_compartments
+from gaphor.UML.classes.stereotype import stereotype_compartments, stereotype_watches
+from gaphor.UML.compartments import name_compartment
 
 
 @represents(UML.Node)
@@ -35,43 +33,22 @@ class NodeItem(Classified, ElementPresentation):
 
         self.watch("children", self.update_shapes)
         self.watch("show_stereotypes", self.update_shapes)
-        self.watch("subject[NamedElement].name")
-        self.watch("subject.appliedStereotype", self.update_shapes)
-        self.watch("subject.appliedStereotype.classifier.name")
-        self.watch("subject.appliedStereotype.slot", self.update_shapes)
-        self.watch("subject.appliedStereotype.slot.definingFeature.name")
-        self.watch("subject.appliedStereotype.slot.value", self.update_shapes)
+        self.watch("subject.name")
         self.watch("subject[Node].ownedConnector", self.update_shapes)
+        stereotype_watches(self)
 
     show_stereotypes: attribute[int] = attribute("show_stereotypes", int)
 
     def update_shapes(self, event=None):
         self.shape = Box(
-            Box(
-                Text(
-                    text=lambda: UML.recipes.stereotypes_str(
-                        self.subject,
-                        isinstance(self.subject, UML.Device)
-                        and (gettext("device"),)
-                        or (),
-                    ),
-                ),
-                Text(
-                    text=lambda: self.subject.name or "",
-                    style={"font-weight": FontWeight.BOLD},
-                ),
-                style={
-                    "padding": (4, 4, 4, 4),
-                    "justify-content": JustifyContent.START,
-                },
+            name_compartment(
+                self,
+                lambda: [self.diagram.gettext("device")]
+                if isinstance(self.subject, UML.Device)
+                else [],
             ),
             *(self.show_stereotypes and stereotype_compartments(self.subject) or []),
-            style={
-                "justify-content": JustifyContent.START
-                if self.diagram and self.children
-                else JustifyContent.CENTER,
-            },
-            draw=draw_node
+            draw=draw_node,
         )
 
 
@@ -92,4 +69,4 @@ def draw_node(box, context, bounding_box):
     cr.move_to(w, 0)
     cr.line_to(w + d, -d)
 
-    stroke(context)
+    stroke(context, fill=True)
